@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.29 2003/11/19 20:38:25 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.30 2003/11/21 21:03:31 fuyu Exp $
  */
 
 #include "config.h"
@@ -142,6 +142,11 @@ typedef struct _RendererContext
   GtkTextTag *response_name_tag;
   GtkTextTag *response_paragraph_tag;
   GtkTextTag *bold_tag;
+
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  regex_t comma_separated_link_pattern;
+  regex_t informal_link_pattern;
+#endif
 } RendererContext;
 
 
@@ -236,12 +241,6 @@ static GQuark popup_context_id;
 static GQuark popup_parent_context_id;
 
 
-#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
-static regex_t comma_separated_link_pattern;
-static regex_t informal_link_pattern;
-#endif
-
-
 void
 prepare_thread_ui_initialization(OchushaApplication *application)
 {
@@ -403,24 +402,6 @@ initialize_thread_ui(OchushaApplication *application)
 #if !ENABLE_MULTIPLE_POPUPS
   initialize_popup_context(&global_popup_context, application);
 #endif
-
-#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
-  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
-  if (regcomp(&comma_separated_link_pattern,
-	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
-	      REG_EXTENDED))
-    {
-      fprintf(stderr, "invalid regular expression\n");
-    }
-
-  /* "\357\274\236\342\211\253"  "" UTF-8ɽ */
-  if (regcomp(&informal_link_pattern,
-	      "[>\357\274\236\342\211\253]+[ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
-	      REG_EXTENDED))
-    {
-      fprintf(stderr, "invalid regular expression\n");
-    }
-#endif
 }
 
 
@@ -660,6 +641,12 @@ popup_context_free(PopupContext *popup_context)
       popup_context->renderer_context.link_text_buffer = NULL;
     }
 
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  regfree(&popup_context->renderer_context.comma_separated_link_pattern);
+  regfree(&popup_context->renderer_context.informal_link_pattern);
+#endif
+
   if (popup_context->url != NULL)
     {
       G_FREE(popup_context->url);
@@ -728,6 +715,24 @@ ensure_popup_context(BBSThreadView *parent, OchushaApplication *application)
 			  (GDestroyNotify)popup_context_free);
 
 
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  if (regcomp(&renderer_context->comma_separated_link_pattern,
+	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      REG_EXTENDED))
+    {
+      fprintf(stderr, "invalid regular expression\n");
+    }
+
+  /* "\357\274\236\342\211\253"  "" UTF-8ɽ */
+  if (regcomp(&renderer_context->informal_link_pattern,
+	      "[>\357\274\236\342\211\253]+[ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      REG_EXTENDED))
+    {
+      fprintf(stderr, "invalid regular expression\n");
+    }
+#endif
+
   return popup_context;
 #else
   return &global_popup_context;
@@ -842,6 +847,24 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       bbs_thread_view_get_tag_by_name(job_args->view, "bold"),
   };
 
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  if (regcomp(&context.comma_separated_link_pattern,
+	      "^[,\357\274\214\343\200\201][ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      REG_EXTENDED))
+    {
+      fprintf(stderr, "invalid regular expression\n");
+    }
+
+  /* "\357\274\236\342\211\253"  "" UTF-8ɽ */
+  if (regcomp(&context.informal_link_pattern,
+	      "[>\357\274\236\342\211\253]+[ ]*([1-9][0-9]*)(-[1-9][0-9]*)?",
+	      REG_EXTENDED))
+    {
+      fprintf(stderr, "invalid regular expression\n");
+    }
+#endif
+
   if (info->offsets == NULL)
     info->offsets = G_NEW0(gint, MAX_RESPONSE);
   else 
@@ -925,6 +948,12 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
     }
   BBS_THREAD_UI_UNLOCK;
 
+#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
+  /* "\357\274\214\343\200\201"  "" UTF-8ɽ */
+  regfree(&context.comma_separated_link_pattern);
+  regfree(&context.informal_link_pattern);
+#endif
+
   if (context.link_text_buffer != default_text_buffer)
     G_FREE(context.link_text_buffer);
   G_FREE(args);
@@ -1375,7 +1404,7 @@ characters_cb(void *user_data, const char *ch, int len)
 	  while (len > 0)
 	    {
 	      while (context->last_text_is_link
-		     && regexec(&comma_separated_link_pattern,
+		     && regexec(&context->comma_separated_link_pattern,
 				text_top, 2, match, 0) == 0)
 		{
 		  int match_len = match[0].rm_eo - match[0].rm_so;
@@ -1438,7 +1467,8 @@ characters_cb(void *user_data, const char *ch, int len)
 	      if (len == 0)
 		break;
 
-	      if (regexec(&informal_link_pattern, text_top, 2, match, 0) == 0)
+	      if (regexec(&context->informal_link_pattern,
+			  text_top, 2, match, 0) == 0)
 		{
 		  int no_match_len = match[0].rm_so;
 		  int match_len = match[0].rm_eo - match[0].rm_so;
