@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.9 2003/06/21 19:20:16 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.10 2003/11/07 14:27:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -54,9 +54,6 @@ static void boardlist_view_init(BoardlistView *view);
 static void boardlist_view_finalize(GObject *object);
 static void boardlist_view_destroy(GtkObject *object);
 
-static gboolean boardlist_view_move_cursor(BoardlistView *view,
-					   GtkMovementStep step, gint count);
-
 static void boardlist_view_do_on_selection(BoardlistView *view,
 					   gint what_to_do);
 static void boardlist_view_do_on_item_at_cursor(BoardlistView *view,
@@ -102,7 +99,7 @@ boardlist_view_get_type(void)
 	  (GInstanceInitFunc)boardlist_view_init,
 	};
 
-      blv_type = g_type_register_static(GTK_TYPE_SCROLLED_WINDOW,
+      blv_type = g_type_register_static(GTK_TYPE_TREE_VIEW,
 					"BoardlistView", &blv_info, 0);
     }
 
@@ -130,7 +127,6 @@ enum {
 
   ADVANCE_VIEW_SIGNAL,
   BACK_VIEW_SIGNAL,
-  MOVE_CURSOR_SIGNAL,
 
   DO_ON_SELECTION_SIGNAL,
   DO_ON_ITEM_AT_CURSOR_SIGNAL,
@@ -139,14 +135,6 @@ enum {
 };
 
 
-enum {
-  TREE_VIEW_MOVE_CURSOR_SIGNAL,
-  TREE_VIEW_UNSELECT_ALL_SIGNAL,
-  TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL,
-  TREE_VIEW_LAST_SIGNAL
-};
-
-
 enum {
   TOGGLE_HIDDEN_STATE,
   KILL_ITEM,
@@ -163,10 +151,9 @@ enum {
 };
 
 
-static GtkScrolledWindowClass *parent_class = NULL;
+static GtkTreeViewClass *parent_class = NULL;
 static gint boardlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark node_info_id;
 
@@ -356,16 +343,6 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 ochusha_marshal_BOOLEAN__OBJECT,
 		 G_TYPE_BOOLEAN, 1,
 		 OCHUSHA_TYPE_BULLETIN_BOARD);
-  boardlist_view_signals[MOVE_CURSOR_SIGNAL] =
-    g_signal_new("wrapped_move_cursor",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-		 G_STRUCT_OFFSET(BoardlistViewClass, move_cursor),
-		 NULL, NULL,
-		 ochusha_marshal_BOOLEAN__ENUM_INT,
-		 G_TYPE_BOOLEAN, 2,
-		 GTK_TYPE_MOVEMENT_STEP,
-		 G_TYPE_INT);
 
   boardlist_view_signals[DO_ON_SELECTION_SIGNAL] =
     g_signal_new("do_on_selection",
@@ -386,12 +363,6 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
 
-  tree_view_signals[TREE_VIEW_MOVE_CURSOR_SIGNAL] =
-    g_signal_lookup("move_cursor", GTK_TYPE_TREE_VIEW);
-  tree_view_signals[TREE_VIEW_UNSELECT_ALL_SIGNAL] =
-    g_signal_lookup("unselect_all", GTK_TYPE_TREE_VIEW);
-  tree_view_signals[TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL] =
-    g_signal_lookup("select_cursor_row", GTK_TYPE_TREE_VIEW);
 
   /* Key bindings */
   gtk_binding_entry_add_signal(binding_set, GDK_o, 0, "do_on_selection", 1,
@@ -411,31 +382,31 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 			       G_TYPE_INT, BACK_VIEW);
 
   gtk_binding_entry_add_signal(binding_set, GDK_Up, 0,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_Down, 0,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, 1);
   gtk_binding_entry_add_signal(binding_set, GDK_p, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_n, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, 1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, 1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD1_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD4_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
 			       
@@ -453,7 +424,6 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 
   klass->advance_view = NULL;
   klass->back_view = NULL;
-  klass->move_cursor = boardlist_view_move_cursor;
 
   klass->do_on_selection = boardlist_view_do_on_selection;
   klass->do_on_item_at_cursor = boardlist_view_do_on_item_at_cursor;
@@ -482,40 +452,21 @@ enum
 static void
 boardlist_view_init(BoardlistView *view)
 {
-  GtkWidget *tree_view;
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
 
   GTK_WIDGET_SET_FLAGS(view, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
 
-  gtk_scrolled_window_set_policy(&view->container,
-				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-
-  tree_view = gtk_tree_view_new();
-  view->view = GTK_TREE_VIEW(tree_view);
-
-  gtk_scrolled_window_set_hadjustment(&view->container, GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)));
-  gtk_scrolled_window_set_vadjustment(&view->container, GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)));
-
-  /* MEMO: Τ褦GtkAdjustmentͿƤʤassertion˰óݤ:
-   * Gtk-CRITICAL **: file gtkrange.c: line 438 (gtk_range_get_adjustment):
-   * assertion 'GTK_IS_RANGE(range)' failed
-   * 2Ф롣
-   */
-  gtk_container_add(GTK_CONTAINER(&view->container), tree_view);
-
-  gtk_widget_show(tree_view);
-
-  view->selection = gtk_tree_view_get_selection(view->view);
+  view->selection = gtk_tree_view_get_selection(&view->view);
   gtk_tree_selection_set_mode(view->selection, GTK_SELECTION_MULTIPLE);
 
-  g_signal_connect(G_OBJECT(view->view), "row-expanded",
+  g_signal_connect(G_OBJECT(view), "row-expanded",
 		   G_CALLBACK(tree_view_row_expanded_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "row-collapsed",
+  g_signal_connect(G_OBJECT(view), "row-collapsed",
 		   G_CALLBACK(tree_view_row_collapsed_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "button_press_event",
+  g_signal_connect(G_OBJECT(view), "button_press_event",
 		   G_CALLBACK(tree_view_button_press_event_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "button_release_event",
+  g_signal_connect(G_OBJECT(view), "button_release_event",
 		   G_CALLBACK(tree_view_button_release_event_cb), view);
   view->last_event_x = 0.0;
   view->last_event_y = 0.0;
@@ -528,7 +479,7 @@ boardlist_view_init(BoardlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_boardlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
 }
 
 
@@ -572,7 +523,7 @@ boardlist_view_expand_category(BoardlistView *view,
     return;
 
   path = gtk_tree_model_get_path(GTK_TREE_MODEL(view->model), &info->iter);
-  expanded = gtk_tree_view_expand_row(view->view, path, FALSE);
+  expanded = gtk_tree_view_expand_row(&view->view, path, FALSE);
 
   gtk_tree_path_free(path);
 
@@ -597,7 +548,7 @@ boardlist_view_collapse_category(BoardlistView *view,
     return;
 
   path = gtk_tree_model_get_path(GTK_TREE_MODEL(view->model), &info->iter);
-  collapsed = gtk_tree_view_collapse_row(view->view, path);
+  collapsed = gtk_tree_view_collapse_row(&view->view, path);
 
   gtk_tree_path_free(path);
 
@@ -609,20 +560,6 @@ boardlist_view_collapse_category(BoardlistView *view,
 }
 
 
-static gboolean
-boardlist_view_move_cursor(BoardlistView *view, GtkMovementStep step,
-			   gint count)
-{
-  gboolean result = FALSE;
-  g_signal_emit(G_OBJECT(view->view),
-		tree_view_signals[TREE_VIEW_MOVE_CURSOR_SIGNAL],
-		0,
-		step, count,
-		&result);
-  return result;
-}
-
-
 enum {
   BOARDLIST_ROW_STATE_HIDDEN = 1,
   BOARDLIST_ROW_STATE_KILLED = 1 << 1,
@@ -664,7 +601,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
       /* FALL THROUGH */
     case POPUP_MENU_OPEN_SELECTION_IN_TAB:
       if (OCHUSHA_IS_BOARD_CATEGORY(data))
-	gtk_tree_view_expand_row(args->view->view, path, TRUE);
+	gtk_tree_view_expand_row(&args->view->view, path, TRUE);
       else
 	g_signal_emit(G_OBJECT(args->view),
 		      boardlist_view_signals[OPEN_VIEW_SIGNAL],
@@ -753,7 +690,7 @@ boardlist_view_do_on_item_at_cursor(BoardlistView *view, gint what_to_do)
   GtkTreePath *path = NULL;
   gpointer data = NULL;
 
-  gtk_tree_view_get_cursor(view->view, &path, NULL);
+  gtk_tree_view_get_cursor(&view->view, &path, NULL);
   if (path == NULL)
     return;
 
@@ -765,15 +702,8 @@ boardlist_view_do_on_item_at_cursor(BoardlistView *view, gint what_to_do)
   if (data != NULL)
     {
       gboolean result = FALSE;
-      g_signal_emit(G_OBJECT(view->view),
-		    tree_view_signals[TREE_VIEW_UNSELECT_ALL_SIGNAL],
-		    0,
-		    &result);
-      g_signal_emit(G_OBJECT(view->view),
-		    tree_view_signals[TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL],
-		    0,
-		    TRUE,
-		    &result);
+      gtk_tree_selection_unselect_all(view->selection);
+      (*parent_class->select_cursor_row)(&view->view, FALSE);
 
       switch (what_to_do)
 	{
@@ -790,7 +720,7 @@ boardlist_view_do_on_item_at_cursor(BoardlistView *view, gint what_to_do)
 	case ADVANCE_VIEW:
 	  if (OCHUSHA_IS_BOARD_CATEGORY(data))
 	    {
-	      if (gtk_tree_view_row_expanded(view->view, path))
+	      if (gtk_tree_view_row_expanded(&view->view, path))
 		boardlist_view_collapse_category(view,
 						 OCHUSHA_BOARD_CATEGORY(data));
 	      else
@@ -861,7 +791,7 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
   view->last_event_y = event->y;
   view->last_event_button = event->button;
 
-  if (!gtk_tree_view_get_path_at_pos(view->view, event->x, event->y, &path,
+  if (!gtk_tree_view_get_path_at_pos(&view->view, event->x, event->y, &path,
 				     NULL, NULL, NULL))
     goto done;
 
@@ -1125,7 +1055,7 @@ boardlist_view_open(BoardlistView *view, GSList *category_list,
 
   g_slist_foreach(category_list, append_category, &args);
 
-  gtk_tree_view_set_model(view->view, GTK_TREE_MODEL(store));
+  gtk_tree_view_set_model(&view->view, GTK_TREE_MODEL(store));
 
   if (view->model != NULL)
     g_object_unref(G_OBJECT(view->model));
