@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.24 2003/11/08 23:43:25 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.25 2003/11/19 20:07:07 fuyu Exp $
  */
 
 #include "config.h"
@@ -494,53 +494,90 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
 		 gboolean in_tab, gboolean use_web_browser)
 {
   OchushaBulletinBoard *board;
-  OchushaBBSThread *thread;
-  unsigned int thread_num;
-  unsigned int from;
-  unsigned int to;
-  char thread_id[PATH_MAX];
+  char *board_url = NULL;
+  OchushaBBSType bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  char *thread_id = NULL;
+  unsigned int from = 0;
+  unsigned int to = 0;
+  char command_line[COMMAND_LINE_BUFFER_SIZE];
 
   /*
    * XXX: Ǥurl2chURLɤĴ٤Ƥۤˤ餷Ƥ뤬
    *      ʥ2ch¸ʬϤ٤
    */
-  if (use_web_browser
-      || !ochusha_utils_2ch_parse_url(application->table, url,
-				      &board, &thread_num, &thread,
-				      &from, &to, NULL))
+  if (!use_web_browser
+      && ochusha_bbs_table_check_url(application->table, url,
+				     &board, &board_url, &bbs_type,
+				     &thread_id, &from, &to))
     {
-      char command_line[COMMAND_LINE_BUFFER_SIZE];
-      if (snprintf(command_line, COMMAND_LINE_BUFFER_SIZE,
-		   application->web_browser_template, url)
-	  < COMMAND_LINE_BUFFER_SIZE)
+#if 0
+      if (board == NULL)
 	{
-	  GError *error = NULL;
-	  /*
-	   * MEMO: UTF-8Ǥ'~'E280BEˤʤäƤꡢ줬ޤޤ줿URL
-	   *       mozillaϤƤŪΥڡɽʤȤ
-	   *       餷(̤ǧδĶǤɽƤ褦ʵ)
-	   *       Τᤳnative localeλꤹencodingѴ롣
-	   *
-	   *       ȥѥå by Motonobu Ichimura <famao@momonga-linux.org>
-	   *
-	   * XXX:  UTF-8ʸnativeencodingѴǽȤϸ¤ʤ
-	   *       Τϥ顼åȤкפ롣
-	   */
-	  char *native_command_line
-	    = g_locale_from_utf8(command_line, -1, NULL, NULL, NULL);
-	  if (!g_spawn_command_line_async(native_command_line, &error))
+	  /* TODO: ɲåɤߤʤΤư*/
+	  fprintf(stderr, "ochusha_open_url()\n");
+	  fprintf(stderr, "  url: %s\n", url);
+	  fprintf(stderr, "  board_url: %s\n", board_url);
+	  fprintf(stderr, "  bbs_type: %d\n", bbs_type);
+	  if (thread_id != NULL)
+	    fprintf(stderr, "  thread_id: %s\n", thread_id);
+	  fprintf(stderr, "  from=%d, to=%d\n", from, to);
+	}
+#endif
+
+      if (board != NULL)
+	{
+	  /* ɽ */
+	  if (thread_id != NULL)
 	    {
-	      /* TODO: 顼ɽ */
-	      fprintf(stderr, "spawning error: %s (%d)\n",
-		      error->message, error->code);
+	      ochusha_open_thread(application, board, thread_id, from, in_tab);
 	    }
-	  g_free(native_command_line);
+	  else
+	    {
+	      paned_notebook_open_item_view(PANED_NOTEBOOK(application->contents_window),
+					    board, in_tab);
+	    }
+
+	  if (board_url != NULL)
+	    G_FREE(board_url);
+	  if (thread_id != NULL)
+	    G_FREE(thread_id);
+	  return;
 	}
-      return;
     }
 
-  snprintf(thread_id, PATH_MAX, "%d.dat", thread_num);
-  ochusha_open_thread(application, board, thread, thread_id, from, in_tab);
+  if (board_url != NULL)
+    G_FREE(board_url);
+  if (thread_id != NULL)
+    G_FREE(thread_id);
+
+  if (snprintf(command_line, COMMAND_LINE_BUFFER_SIZE,
+	       application->web_browser_template, url)
+      < COMMAND_LINE_BUFFER_SIZE)
+    {
+      GError *error = NULL;
+      /*
+       * MEMO: UTF-8Ǥ'~'E280BEˤʤäƤꡢ줬ޤޤ줿URL
+       *       mozillaϤƤŪΥڡɽʤȤ
+       *       餷(̤ǧδĶǤɽƤ褦ʵ)
+       *       Τᤳnative localeλꤹencodingѴ롣
+       *
+       *       ȥѥå by Motonobu Ichimura <famao@momonga-linux.org>
+       *
+       * XXX:  UTF-8ʸnativeencodingѴǽȤϸ¤ʤ
+       *       Τϥ顼åȤкפ롣
+       */
+      char *native_command_line
+	= g_locale_from_utf8(command_line, -1, NULL, NULL, NULL);
+      if (!g_spawn_command_line_async(native_command_line, &error))
+	{
+	  /* TODO: 顼ɽ */
+	  fprintf(stderr, "spawning error: %s (%d)\n",
+		  error->message, error->code);
+	}
+      g_free(native_command_line);
+    }
+
+  return;
 }
 
 
