@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.19 2003/11/20 03:14:37 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.20 2003/11/21 21:03:31 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -41,29 +41,6 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
-#  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
-#  endif
-# else
-#  if HAVE_POSIX_REGEX
-#    if HAVE_SYS_TYPES_H
-#      include <sys/types.h>
-#    endif
-#    if HAVE_REGEX_H
-#      include <regex.h>
-#    else
-#      warning "I don't know suitable header file of your posix regex library."
-#    endif
-#  endif
-# endif
-#endif
-
 
 #define DEBUG_GUI		0
 #define DEBUG_GUI_MOST		0
