@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.3 2003/06/06 19:37:23 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.4 2003/11/07 14:27:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -927,14 +927,6 @@ paned_notebook_get_selector(PanedNotebook *paned_notebook)
 }
 
 
-static void
-open_view_cb(GtkWidget *selector, gpointer item, gboolean in_tab,
-	     PanedNotebook *paned_notebook)
-{
-  paned_notebook_open_item_view(paned_notebook, item, in_tab);
-}
-
-
 void
 paned_notebook_set_selector(PanedNotebook *paned_notebook, GtkWidget *selector)
 {
@@ -947,8 +939,6 @@ paned_notebook_set_selector(PanedNotebook *paned_notebook, GtkWidget *selector)
 
   if (selector != NULL)
     {
-      g_signal_connect(G_OBJECT(selector), "open_view",
-		       G_CALLBACK(open_view_cb), paned_notebook);
       gtk_paned_add1(GTK_PANED(&paned_notebook->container), selector);
     }
 
