@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.h,v 1.10 2003/11/08 23:44:33 fuyu Exp $
+ * $Id: ochusha_bbs_thread.h,v 1.12 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_THREAD_H_
@@ -71,7 +71,7 @@ struct _OchushaBBSThread
 
   OchushaBulletinBoard *board;
 
-  char *id;	      /* 㤨DATե̾ */
+  char *id;	      /* 㤨DATե̾"NNNN.dat""NNNN" */
   gchar *title;       /* "顪 GTK+ȤäLinuxǤ塼" */
 
   /* оΥ쥹(㤨subject.txt) */
@@ -115,7 +115,7 @@ struct _OchushaBBSThreadClass
 			      gpointer callback_data);
   time_t (*get_last_modified_utc)(OchushaBBSThread *thread);
   const char *(*get_url)(OchushaBBSThread *thread);
-  char *(*get_url_for_response)(OchushaBBSThread *thread, int number);
+  char *(*get_url_for_response)(OchushaBBSThread *thread, int from, int to);
   const char *(*get_url_to_post)(OchushaBBSThread *thread);
   gboolean (*check_url)(OchushaBBSThread *thread, const char *url,
 			unsigned int *from_p, unsigned int *to_p);
@@ -127,6 +127,7 @@ struct _OchushaBBSThreadClass
 			       EachResponseCallback *response_cb,
 			       EndThreadCallback *end_cb,
 			       gpointer callback_data);
+  gboolean (*post_supported)(OchushaBBSThread *thread);
   gboolean (*post_response)(OchushaBBSThread *thread, OchushaConfig *config,
 			    const OchushaBBSResponse *response);
 };
@@ -169,7 +170,8 @@ int ochusha_bbs_thread_get_number_of_responses_read(OchushaBBSThread *thread);
 time_t ochusha_bbs_thread_get_last_modified_utc(OchushaBBSThread *thread);
 
 const char *ochusha_bbs_thread_get_url(OchushaBBSThread *thread);
-char *ochusha_bbs_thread_get_url_for_response(OchushaBBSThread *thread, int number);
+char *ochusha_bbs_thread_get_url_for_response(OchushaBBSThread *thread,
+					      int from, int to);
 
 gboolean ochusha_bbs_thread_is_preview_supported(OchushaBBSThread *thread);
 gboolean ochusha_bbs_thread_is_post_supported(OchushaBBSThread *thread);
