@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.13 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.14 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -1034,9 +1034,17 @@ ochusha_bulletin_board_read_threadlist_xml(OchushaBulletinBoard *board,
   g_return_val_if_fail(OCHUSHA_IS_BULLETIN_BOARD(board) && config != NULL,
 		       FALSE);
 
-  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s",
-	       board->server, board->base_path, OCHUSHA_THREADLIST_XML)
-      >= PATH_MAX)
+#if 0
+  fprintf(stderr, "server: %s\n", ochusha_bulletin_board_get_server(board));
+  fprintf(stderr, "base_path: %s\n", ochusha_bulletin_board_get_base_path(board));
+  fprintf(stderr, "id: %s\n", ochusha_bulletin_board_get_id(board));
+#endif
+
+  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
+	       ochusha_bulletin_board_get_server(board),
+	       ochusha_bulletin_board_get_base_path(board),
+	       ochusha_bulletin_board_get_id(board),
+	       OCHUSHA_THREADLIST_XML) >= PATH_MAX)
     return FALSE;
 
   threadlist_xml = ochusha_config_find_file(config, pathname);
@@ -1122,9 +1130,11 @@ ochusha_bulletin_board_write_threadlist_xml(OchushaBulletinBoard *board,
 		       FALSE);
   g_return_val_if_fail(config->home != NULL, FALSE);
 
-  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s",
-	       board->server, board->base_path, OCHUSHA_THREADLIST_XML)
-      >= PATH_MAX)
+  if (snprintf(pathname, PATH_MAX, "cache/%s%s%s/%s",
+	       ochusha_bulletin_board_get_server(board),
+	       ochusha_bulletin_board_get_base_path(board),
+	       ochusha_bulletin_board_get_id(board),
+	       OCHUSHA_THREADLIST_XML) >= PATH_MAX)
     return FALSE;
 
   fd = ochusha_config_open_file(config, pathname,
