@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.7 2003/11/19 22:49:41 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.8 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -79,7 +79,7 @@ struct _OchushaBulletinBoard
 
   /* ʲλĤbase_url롣*/
   char *server;			/* 㤨 "pc.2ch.net" */
-  char *base_path;		/* 㤨 "/unix/" */
+  char *base_path;		/* 㤨 "/" */
   char *id;			/* 㤨 "unix" */
 
   /* JBBSФĤξbase_pathidϼΤ褦ˤʤ롣
