@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.10 2003/11/19 21:37:59 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.11 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -61,7 +61,7 @@
 #define DISABLE_PERSISTENT_CONNECTION		0
 #define ENABLE_GZIPPED_DIFFERENCIAL_READ	0
 
-#define CACHE_COMPARE_SIZE	512	/* Ĥѹǽˤ٤ */
+#define CACHE_COMPARE_SIZE	128	/* Ĥѹǽˤ٤ */
 #define HEADER_BUFFER_LENGTH	256
 
 #define POLLING_INTERVAL_SEC	0
@@ -1936,7 +1936,26 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
     }
 
   status_code = ghttp_status_code(request);
-  if (status_code != 200)
+  status->status_code = status_code;
+
+  if (status_code == 200)
+    {
+      const char *tmp_header
+	= ghttp_get_header(request, http_hdr_Last_Modified);
+      if (tmp_header != NULL)
+	status->last_modified = G_STRDUP(tmp_header);
+      else
+	status->last_modified = NULL;
+      tmp_header = ghttp_get_header(request, http_hdr_Date);
+      if (tmp_header != NULL)
+	status->date = G_STRDUP(tmp_header);
+      else
+	status->date = NULL;
+#if DEBUG_NETWORK
+      fprintf(stderr, "Synchronous read done.\n");
+#endif
+    }
+  else
     {
       goto error_exit;
     }
@@ -2037,6 +2056,11 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #if DEBUG_NETWORK_MOST
   fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
 #endif
+  g_signal_emit(G_OBJECT(broker),
+		broker_signals[ACCESS_COMPLETED_SIGNAL],
+		0,
+		buffer,
+		&signal_result);
 
   return TRUE;
 
