@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.h,v 1.2 2003/11/02 22:01:15 fuyu Exp $
+ * $Id: ochusha_network_broker.h,v 1.3 2003/11/18 21:02:05 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_NETWORK_BROKER_H_
@@ -132,6 +132,9 @@ OchushaAsyncBuffer *ochusha_network_broker_read_from_url(
 					const char *url,
 					const char *if_modified_since,
 					OchushaNetworkBrokerCacheMode mode);
+OchushaAsyncBuffer *ochusha_network_broker_read_from_cgi(
+					OchushaNetworkBroker *broker,
+					const char *url);
 
 const char *ochusha_network_broker_get_response_header(
 						  OchushaNetworkBroker *broker,
