@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_private.h,v 1.3 2003/07/15 05:37:42 fuyu Exp $
+ * $Id: ochusha_private.h,v 1.5 2003/11/04 07:56:41 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_PRIVATE_H_
@@ -37,6 +37,8 @@
 #define ENABLE_SKIP_IME_NU	1
 #define ENABLE_MS_IZONMOJI	1
 
+#define DEBUG_SAX_HANDLER	0
+#define DEBUG_SAX_HANDLER_NEW	0
 
 #define DEBUG_PARSER		0
 #define DEBUG_PARSER_MOST	0
@@ -44,6 +46,9 @@
 #define DEBUG_ICONV_MOST	0
 #define DEBUG_UTILS		0
 
+#define DEBUG_POST		1
+#define DEBUG_POST_MOST		0
+
 
 #include "utils.h"
 
