@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.23 2003/11/20 03:14:37 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.24 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -33,6 +33,7 @@
 #include "ochusha_board_2ch.h"
 #include "ochusha_thread_2ch.h"
 #include "ochusha_utils_2ch.h"
+#include "ochusha_utils_jbbs.h"
 
 #include <glib-object.h>
 #include <glib.h>
@@ -43,30 +44,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
-#  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
-#  endif
-# else
-#  if HAVE_POSIX_REGEX
-#    if HAVE_SYS_TYPES_H
-#      include <sys/types.h>
-#    endif
-#    if HAVE_REGEX_H
-#      include <regex.h>
-#    else
-#      warning "I don't know suitable header file of your posix regex library."
-#    endif
-#  endif
-# endif
-# include "ochusha_utils_jbbs.h"
-#endif
-
 
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
@@ -159,10 +136,6 @@ ochusha_thread_2ch_get_type(void)
 
 static OchushaBBSThreadClass *parent_class = NULL;
 
-#if ENABLE_REGEXP
-static regex_t cgi_query_pattern;
-#endif
-
 static void
 ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
 {
@@ -190,11 +163,6 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
   b_class->preview_response = ochusha_thread_2ch_preview_response;
   b_class->post_supported = ochusha_thread_2ch_post_supported;
   b_class->post_response = ochusha_thread_2ch_post_response;
-
-#if ENABLE_REGEXP
-  if (regcomp(&cgi_query_pattern, "&?([^=]+)=([^&]*)", REG_EXTENDED))
-    fprintf(stderr, "invalid regular expression\n");
-#endif
 }
 
 
@@ -345,254 +313,35 @@ ochusha_thread_2ch_property_change_notify(OchushaThread2ch *thread)
 }
 
 
-#if ENABLE_REGEXP
-/*
- * "../test/read.cgi?bbs=linux&key=987524960&st=109&to=109&nofirst=true"
- * ߤURLβϡ
- */
-static gboolean
-ochusha_thread_2ch_check_relative_url(OchushaThread2ch *thread_2ch,
-				      const char *url,
-				      unsigned int *from_p, unsigned int *to_p)
-{
-  unsigned int from = 0;
-  unsigned int to = 0;
-  gboolean is_same_board = FALSE;
-  gboolean is_same_thread = FALSE;
-  OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
-  regmatch_t match[3];
-
-  url = strchr(url, '?');
-  if (url == NULL)
-    return FALSE;
-  url++;
-
-  /* cgi_query_pattern"^([^=]+)=([^&]*)"ɽ */
-  while (regexec(&cgi_query_pattern, url, 2, match, 0) == 0)
-    {
-      int match_len = match[0].rm_eo - match[0].rm_so;
-      const char *name_head = url + match[1].rm_so;
-      int name_len = match[1].rm_eo - match[1].rm_so;
-      const char *value_head = url + match[2].rm_so;
-      int value_len = match[2].rm_eo - match[2].rm_so;
-
-      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
-	{
-	  if (strncmp(thread->board->id, value_head, value_len) == 0)
-	    is_same_board = TRUE;
-	  else
-	    return FALSE;
-	}
-      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
-	{
-	  const char *key = thread->id;
-	  if (strncmp(key, value_head, value_len) == 0)
-	    is_same_thread = TRUE;
-	  else
-	    return FALSE;
-	}
-      else if (g_ascii_strncasecmp("ST", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("START", name_head, name_len) == 0)
-	{
-	  if (value_len > 0)
-	    {
-	      char *value = g_strndup(value_head, value_len);
-	      if (sscanf(value, "%d", &from) != 1)
-		from = 0;
-	      g_free(value);
-	    }
-	}
-      else if (g_ascii_strncasecmp("TO", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("END", name_head, name_len) == 0)
-	{
-	  if (value_len > 0)
-	    {
-	      char *value = g_strndup(value_head, value_len);
-	      if (sscanf(value, "%d", &to) != 1)
-		to = 0;
-	      g_free(value);
-	    }
-	}
-
-      url += match_len;
-    }
-
-  if (!is_same_board || !is_same_thread)
-    return FALSE;
-
-  if (from == 0 && to != 0)
-    from = to;
-  else if (to == 0 && from != 0)
-    to = from;
-  else if (from == 0 && to == 0)
-    {
-      from = 1;
-      to = 1;
-    }
-
-  if (from_p != NULL)
-    *from_p = from;
-
-  if (to_p != NULL)
-    *to_p = to;
-
-  return TRUE;
-}
-
-
-/*
- * "http://jbbs.shitaraba.com/computer/bbs/read.cgi?BBS=351&KEY=1040452877&START=109&END=109&NOFIRST=TRUE"
- * ߤURLβϡ
- */
-static gboolean
-ochusha_thread_2ch_check_absolute_url(OchushaThread2ch *thread_2ch,
-				      const char *url,
-				      unsigned int *from_p, unsigned int *to_p)
-{
-  unsigned int from = 0;
-  unsigned int to = 0;
-  gboolean is_same_board = FALSE;
-  gboolean is_same_thread = FALSE;
-  OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
-  regmatch_t match[3];
-
-  url = strchr(url, '?');
-  if (url == NULL)
-    return FALSE;
-  url++;	/* ?ޤǤФ */
-
-  /* cgi_query_pattern"^([^=]+)=([^&]*)"ɽ */
-  while (regexec(&cgi_query_pattern, url, 2, match, 0) == 0)
-    {
-      int match_len = match[0].rm_eo - match[0].rm_so;
-      const char *name_head = url + match[1].rm_so;
-      int name_len = match[1].rm_eo - match[1].rm_so;
-      const char *value_head = url + match[2].rm_so;
-      int value_len = match[2].rm_eo - match[2].rm_so;
-
-      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
-	{
-	  if (strncmp(thread->board->id, value_head, value_len) == 0)
-	    is_same_board = TRUE;
-	  else
-	    return FALSE;
-	}
-      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
-	{
-	  if (strncmp(thread->id, value_head, value_len) == 0)
-	    is_same_thread = TRUE;
-	  else
-	    return FALSE;
-	}
-      else if (g_ascii_strncasecmp("START", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("ST", name_head, name_len) == 0)
-	{
-	  if (value_len > 0)
-	    {
-	      char *value = g_strndup(value_head, value_len);
-	      if (sscanf(value, "%d", &from) != 1)
-		from = 0;
-	      g_free(value);
-	    }
-	}
-      else if (g_ascii_strncasecmp("END", name_head, name_len) == 0
-	       || g_ascii_strncasecmp("TO", name_head, name_len) == 0)
-	{
-	  if (value_len > 0)
-	    {
-	      char *value = g_strndup(value_head, value_len);
-	      if (sscanf(value, "%d", &to) != 1)
-		to = 0;
-	      g_free(value);
-	    }
-	}
-
-      url += match_len;
-    }
-
-  if (!is_same_board || !is_same_thread)
-    return FALSE;
-
-  if (from == 0 && to != 0)
-    from = to;
-  else if (to == 0 && from != 0)
-    to = from;
-  else if (from == 0 && to == 0)
-    {
-      from = 1;
-      to = 1;
-    }
-
-  if (from_p != NULL)
-    *from_p = from;
-
-  if (to_p != NULL)
-    *to_p = to;
-
-  return TRUE;
-}
-#endif
-
-
 static gboolean
 ochusha_thread_2ch_check_url(OchushaBBSThread *thread, const char *url,
 			     unsigned int *from_p, unsigned int *to_p)
 {
-  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), FALSE);
-  if (strncmp(url, "../", 3) == 0)
-    {
-      int len;
-      const char *base_path;
-      unsigned int from = 0;
-      unsigned int to = 0;
-      OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
+  char *board_id = NULL;
+  char *thread_id = NULL;
+  gboolean result;
 
-      while (strncmp(url + 3, "../", 3) == 0)
-	url += 3;	/* ƬϢ³"../"ɤФ */
-
-      base_path = ochusha_thread_2ch_get_base_path(thread_2ch);
-      g_return_val_if_fail(base_path != NULL, FALSE);
-
-      len = strlen(base_path);
-      url += 2;	/* Ƭ".."ɤФ */
+  if (!ochusha_utils_2ch_check_url(url, NULL, NULL, &board_id,
+				   &thread_id, from_p, to_p))
+    return FALSE;
 
 #if 0
-      fprintf(stderr, "ochusha_thread_2ch_check_url()\n");
-      fprintf(stderr, "  url: %s\n", url);
-      fprintf(stderr, "  base_path: %s\n", base_path);
+  fprintf(stderr, "check_url: board_id=\"%s\" thread_id=\"%s\"\n",
+	  board_id, thread_id);
 #endif
-
-      if (strncmp(base_path, url, len) != 0)
-#if ENABLE_REGEXP
-	return ochusha_thread_2ch_check_relative_url(thread_2ch, url - 2,
-						     from_p, to_p);
-#else
-	return FALSE;
-#endif
-
-      sscanf(url + len, "%u-%u", &from, &to);
-      if (from < 0)
-	{
-	  to = -from;
-	  from = 0;
-	}
-      else if (to < 0)
-	to = -to;
-
-      if (from_p != NULL)
-	*from_p = from;
-      if (to_p != NULL)
-	*to_p = to;
-
-      return TRUE;
-    }
-#if ENABLE_REGEXP
+  if (strcmp(ochusha_bulletin_board_get_id(thread->board), board_id) == 0
+      && strcmp(thread->id, thread_id) == 0)
+    result = TRUE;
   else
-    return ochusha_thread_2ch_check_absolute_url(OCHUSHA_THREAD_2CH(thread),
-						 url, from_p, to_p);
-#endif
+    result = FALSE;
 
-  return FALSE;
+  if (board_id != NULL)
+    G_FREE(board_id);
+
+  if (thread_id != NULL)
+    G_FREE(thread_id);
+
+  return result;
 }
 
 
@@ -652,6 +401,7 @@ ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread_2ch)
 {
   char url[PATH_MAX];
   const char *base_path;
+  const char *server;
   OchushaBBSThread *thread;
 
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
@@ -662,12 +412,10 @@ ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread_2ch)
   base_path = ochusha_thread_2ch_get_base_path(thread_2ch);
 
   thread = OCHUSHA_BBS_THREAD(thread_2ch);
+  server = ochusha_bulletin_board_get_server(thread->board);
 
-  g_return_val_if_fail(base_path != NULL &&
-		       thread->board != NULL && thread->board->server != NULL,
-		       NULL);
-  if (snprintf(url, PATH_MAX, "http://%s%s",
-	       thread->board->server, base_path) >= PATH_MAX)
+  g_return_val_if_fail(base_path != NULL && server != NULL, NULL);
+  if (snprintf(url, PATH_MAX, "http://%s%s", server, base_path) >= PATH_MAX)
     return NULL;
 
   thread_2ch->base_url = G_STRDUP(url);
@@ -694,8 +442,9 @@ ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-      if (snprintf(base_path, PATH_MAX, "/test/read.cgi%s%s/",
+      if (snprintf(base_path, PATH_MAX, "/test/read.cgi%s%s/%s/",
 		   ochusha_bulletin_board_get_base_path(thread->board),
+		   ochusha_bulletin_board_get_id(thread->board),
 		   thread->id) >= PATH_MAX)
 	return NULL;
       break;
@@ -750,16 +499,13 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
       return ochusha_network_broker_read_from_url(broker,
 				ochusha_thread_2ch_get_dat_url(thread_2ch),
 				thread_2ch->last_modified, mode);
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
+
     case OCHUSHA_BBS_TYPE_JBBS:
     case OCHUSHA_BBS_TYPE_MACHIBBS:
     case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
     case OCHUSHA_BBS_TYPE_MITINOKU:
       return ochusha_utils_jbbs_fake_responses_source(thread_2ch, broker,
 						      mode);
-# endif
-#endif
     default:
       return NULL;
     }
@@ -1278,8 +1024,7 @@ ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread, int from,
       {
 	if (from >= to)
 	  to = from;
-	snprintf(url, PATH_MAX, "http://%s/read.cgi?bbs=%s&key=%s&st=%d&to=%d&nofirst=true",
-		 ochusha_bulletin_board_get_server(thread->board),
+	snprintf(url, PATH_MAX, "./read.cgi?bbs=%s&key=%s&st=%d&to=%d&nofirst=true",
 		 ochusha_bulletin_board_get_id(thread->board),
 		 thread->id, from, to);
 	break;
