@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.11 2003/11/18 21:02:05 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.12 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,29 @@
 
 #include <glib.h>
 
+#if ENABLE_REGEXP
+# if HAVE_ONIGURUMA
+#  if HAVE_ONIG_ONIGPOSIX_H
+#    include <onig/onigposix.h>
+#  elif HAVE_ONIGPOSIX_H
+#    include <onigposix.h>
+#  else
+#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
+#  endif
+# else
+#  if HAVE_POSIX_REGEX
+#    if HAVE_SYS_TYPES_H
+#      include <sys/types.h>
+#    endif
+#    if HAVE_REGEX_H
+#      include <regex.h>
+#    else
+#      warning "I don't know suitable header file of your posix regex library."
+#    endif
+#  endif
+# endif
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,6 +70,569 @@
 #define GUESS_2CH_BOARD_URL	1
 
 
+static gboolean
+parse_thread_path(char *path, char **thread_id_p,
+		  unsigned int *from_p, unsigned int *to_p)
+{
+  char *thread_id = NULL;
+  unsigned int from = 0;
+  unsigned int to = 0;
+  char *slash_pos = strchr(path + 1, '/');
+
+#if 0
+  fprintf(stderr, "parse_thread_path()\n");
+  fprintf(stderr, "  path: %s\n", path);
+#endif
+
+  if (slash_pos != NULL)
+    {
+      if (thread_id_p != NULL)
+	thread_id = G_STRNDUP(path, slash_pos - path);
+      else
+	thread_id = path;
+      path = slash_pos + 1;
+
+      if (*path != '\0')
+	{
+	  char tmp_char;
+	  char *hyphen_pos = strchr(path, '-');
+	  if (hyphen_pos != NULL)
+	    {
+	      tmp_char = *hyphen_pos;
+	      *hyphen_pos = '\0';
+	    }
+	  else
+	    tmp_char = '\0';
+
+	  if (hyphen_pos != path)
+	    {
+	      if (sscanf(path, "%d", &from) != 1)
+		from = 0;
+	    }
+
+	  if (hyphen_pos != NULL && hyphen_pos[1] != '\0')
+	    {
+	      if (sscanf(hyphen_pos + 1, "%d", &to) != 1)
+		to = from;
+	    }
+	}
+    }
+  else
+    {
+      if (thread_id_p != NULL && *path != '\0')
+	thread_id = G_STRDUP(path);
+      else
+	thread_id = path;
+    }
+
+  if (thread_id_p != NULL)
+    *thread_id_p = thread_id;
+
+  if (from_p != NULL)
+    *from_p = from;
+
+  if (to_p != NULL)
+    *to_p = to;
+
+#if 0
+  fprintf(stderr, "  thread_id: %s\n", thread_id);
+  fprintf(stderr, "  from=%d, to=%d\n", from, to);
+#endif
+
+  return thread_id != NULL;
+}
+
+
+#if ENABLE_REGEXP
+static gboolean
+parse_cgi_query(const char *query, char **board_id_p, char **thread_id_p,
+		unsigned int *from_p, unsigned int *to_p)
+{
+  char board_id[PATH_MAX];
+  char thread_id[PATH_MAX];
+  char tmp_buf[PATH_MAX];
+  unsigned int from = 0;
+  unsigned int to = 0;
+  regex_t cgi_query_pattern;
+  regmatch_t match[3];
+  char *tmp_pos;
+
+  if (regcomp(&cgi_query_pattern, "&?([^=]+)=([^&]*)", REG_EXTENDED))
+    {
+      fprintf(stderr, "invalid_regular expression\n");
+      return FALSE;
+    }
+#if 0
+  fprintf(stderr, "parse_cgi_query()\n");
+  fprintf(stderr, "  query: %s\n", query);
+#endif
+
+  board_id[0] = '\0';
+  thread_id[0] = '\0';
+
+  tmp_pos = strchr(query, '?');
+  if (tmp_pos != NULL)
+    query = tmp_pos + 1;	/* ?ޤǤФ */
+
+  while (regexec(&cgi_query_pattern, query, 2, match, 0) == 0)
+    {
+      int match_len = match[0].rm_eo - match[0].rm_so;
+      const char *name_head = query + match[1].rm_so;
+      int name_len = match[1].rm_eo - match[1].rm_so;
+      const char *value_head = query + match[2].rm_so;
+      int value_len = match[2].rm_eo - match[2].rm_so;
+
+      if (g_ascii_strncasecmp("BBS", name_head, name_len) == 0)
+	{
+	  if (value_len < (PATH_MAX - 1))
+	    {
+	      memcpy(board_id, value_head, value_len);
+	      board_id[value_len] = '\0';
+	    }
+	}
+      else if (g_ascii_strncasecmp("KEY", name_head, name_len) == 0)
+	{
+	  if (value_len < (PATH_MAX - 1))
+	    {
+	      memcpy(thread_id, value_head, value_len);
+	      thread_id[value_len] = '\0';
+	    }
+	}
+      else if (g_ascii_strncasecmp("START", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("ST", name_head, name_len) == 0)
+	{
+	  if (value_len > 0 && value_len < (PATH_MAX - 1))
+	    {
+	      memcpy(tmp_buf, value_head, value_len);
+	      tmp_buf[value_len] = '\0';
+	      if (sscanf(tmp_buf, "%d", &from) != 1)
+		from = 0;
+	    }
+	}
+      else if (g_ascii_strncasecmp("END", name_head, name_len) == 0
+	       || g_ascii_strncasecmp("TO", name_head, name_len) == 0)
+	{
+	  if (value_len > 0 && value_len < (PATH_MAX - 1))
+	    {
+	      memcpy(tmp_buf, value_head, value_len);
+	      tmp_buf[value_len] = '\0';
+	      if (sscanf(tmp_buf, "%d", &to) != 1)
+		to = 0;
+	    }
+	}
+
+      query += match_len;
+    }
+
+  if (from == 0 && to != 0)
+    from = to;
+  else if (to == 0 && from != 0)
+    to = from;
+#if 0
+  else if (from == 0 && to == 0)
+    {
+      from = 1;
+      to = 1;
+    }
+#endif
+
+  if (from_p != NULL)
+    *from_p = from;
+
+  if (to_p != NULL)
+    *to_p = to;
+
+  if (board_id[0] == '\0')
+    thread_id[0] = '\0';
+
+  if (board_id_p != NULL)
+    {
+      if (board_id[0] != '\0')
+	*board_id_p = G_STRDUP(board_id);
+      else
+	*board_id_p = NULL;
+    }
+
+  if (thread_id_p != NULL)
+    {
+      if (thread_id[0] != '\0')
+	*thread_id_p = G_STRDUP(thread_id);
+      else
+	*thread_id_p = NULL;
+    }
+
+#if 0
+  fprintf(stderr, "  board_id: %s\n", board_id);
+  fprintf(stderr, "  thread_id: %s\n", thread_id);
+  fprintf(stderr, "  from=%d, to=%d\n", from, to);
+#endif
+
+  regfree(&cgi_query_pattern);
+
+  return board_id[0] != '\0';
+}
+#endif
+
+
+/*
+ * Ϳ줿URLϤбƤʷǼĤΤΤɤ
+ * Ƚꤹ롣
+ *
+ * Ƚ˲äʲΤ褦Ѥ򵯤
+ *
+ * ȤͿ줿board_url_pNULLʾ:
+ *   URLǼ줿ڡбǽʷǼĤΤΤäݤ
+ *   tableˤ̤Ͽʾ硢ĤURLȤ餷ʸΥԡ
+ *   board_url_p˳Ǽ롣
+ * ȤͿ줿bbs_type_pNULLʾ:
+ *   URLǼ줿ڡбǽʷǼĤΤΤäݤ
+ *   tableˤ̤Ͽʾ硢ļ̤Ȥ餷ͤ
+ *   bbs_type_p˳Ǽ롣
+ * ȤͿ줿thread_id_pfrom_pto_pNULLʾ:
+ *   URLǼ줿ڡбǽʷǼĤΤΤäݤ硢
+ *   ID(thread_id)䡢쥹ϰϻƬ(from)(to)򤽤줾
+ *   thread_id_pfrom_pto_p˳Ǽ롣thread_id_p˳ǼΤ
+ *   IDʸΥԡ
+ */
+gboolean
+ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
+			    OchushaBBSType *bbs_type_p,
+			    char **board_id_p, char **thread_id_p,
+			    unsigned int *from_p, unsigned int *to_p)
+{
+  char *server;
+  char *abs_path;
+  char *path_pos;
+  OchushaBBSType bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+  char *thread_id = NULL;
+  char *board_id = NULL;
+  int from = 0;
+  int to = 0;
+  char board_url[PATH_MAX];
+  char tmp_buf[PATH_MAX];
+
+  g_return_val_if_fail(url != NULL, FALSE);
+
+  board_url[0] = '\0';
+
+  server = ochusha_utils_url_extract_http_server(url);
+  if (server != NULL)
+    abs_path = ochusha_utils_url_extract_http_absolute_path(url);
+  else if (board_url_p == NULL)
+    {
+      const char *tmp_pos = url;
+      server = G_STRDUP("localhost");	/*  */
+      while (TRUE)
+	{
+	  if (strncmp(tmp_pos, "../", 3) == 0)
+	    {
+	      tmp_pos += 2;
+	      continue;
+	    }
+	  if (strncmp(tmp_pos, "/../", 4) == 0)
+	    {
+	      tmp_pos += 3;
+	      continue;
+	    }
+	  if (strncmp(tmp_pos, "./", 2) == 0)
+	    {
+	      tmp_pos ++;
+	      continue;
+	    }
+	  if (strncmp(tmp_pos, "/./", 3) == 0)
+	    {
+	      tmp_pos += 2;
+	      continue;
+	    }
+	  break;
+	}
+      abs_path = G_STRDUP(tmp_pos);
+    }
+  else
+    abs_path = NULL;
+
+#if 0
+  fprintf(stderr, "ochusha_utils_2ch_check_url()\n");
+  fprintf(stderr, "  url: %s\n", url);
+  fprintf(stderr, "  server: %s\n", server);
+  fprintf(stderr, "  abs_path: %s\n", abs_path);
+#endif
+
+  if (server == NULL || abs_path == NULL || *abs_path != '/')
+    goto done;
+
+  path_pos = abs_path + 1;
+  if (strncmp(path_pos, "test/read.cgi", 13) == 0)
+    {
+      bbs_type = OCHUSHA_BBS_TYPE_2CH;
+      if (path_pos[13] == '/')
+	{
+	  char *slash_pos = strchr(path_pos + 14, '/');
+	  if (slash_pos != NULL)
+	    {
+	      char tmp_char = slash_pos[1];
+	      slash_pos[1] = '\0';
+	      if (snprintf(board_url, PATH_MAX, "http://%s%s", server,
+			   path_pos + 13) < PATH_MAX)
+		{
+#if 0
+		  fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+
+		  slash_pos[1] = tmp_char;
+		  tmp_char = slash_pos[0];
+		  slash_pos[0] = '\0';
+		  board_id = G_STRDUP(path_pos + 14);
+		  slash_pos[0] = tmp_char;
+
+		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+		}
+	      else
+		board_url[0] = '\0';
+	    }
+	}
+#if ENABLE_REGEXP
+      else if (path_pos[13] == '?')
+	{
+	  if (parse_cgi_query(path_pos + 14, &board_id, &thread_id,
+			      &from, &to))
+	    {
+	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
+			   board_id) >= PATH_MAX)
+		board_url[0] = '\0';
+#if 0
+	      fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+	    }
+	}
+#endif
+    }
+  else if (g_ascii_strcasecmp(server, "jbbs.shitaraba.com") == 0)
+    {
+      /* JBBSФ̯ʴ*/
+      char *tmp_pos = strstr(path_pos, "bbs/read.cgi");
+      bbs_type = OCHUSHA_BBS_TYPE_JBBS_SHITARABA;
+      if (tmp_pos == NULL)
+	{
+	  /* URLȤƤ
+	   * http://jbbs.shitaraba.com/computer/351/
+	   * ߤʤΤ⤢롣
+	   */
+	  tmp_pos = strchr(path_pos, '/');
+	  if (tmp_pos != NULL)
+	    {
+	      char *board_id_head = tmp_pos + 1;
+	      tmp_pos = strchr(board_id_head, '/');
+	      if (tmp_pos != NULL)
+		{
+		  char tmp_char = tmp_pos[1];
+		  tmp_pos[1] = '\0';
+		  if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
+			       path_pos) < PATH_MAX)
+		    board_id = G_STRDUP(board_id_head);
+		  else
+		    board_url[0] = '\0';
+		  tmp_pos[1] = tmp_char;
+		}
+	    }
+	}
+      else
+	{
+	  /*
+	   * URLȤƤϰʲ2ĤηΤʤ¾ˤ⤢뤫⡣
+	   */
+	  if (tmp_pos[12] == '/')
+	    {
+	      /*
+	       * http://jbbs.shitaraba.com/bbs/read.cgi/computer/351/1040452877/109
+	       * URL
+	       */
+	      char *slash_pos = strchr(tmp_pos + 1, '/');
+	      if (slash_pos != NULL)
+		{
+		  char *board_id_head = slash_pos + 1;
+		  slash_pos = strchr(board_id_head, '/'); /* 2ܤ'/'õ*/
+		  if (slash_pos != NULL)
+		    {
+		      /* (tmp_pos + 12)slash_posޤǤʸ
+		       * "/computer/351/"ʬˤʤ
+		       */
+		      *slash_pos = '\0';
+		      if (snprintf(board_url, PATH_MAX, "http://%s%s/", server,
+				   tmp_pos + 12) < PATH_MAX)
+			{
+			  board_id = G_STRDUP(board_id_head);
+			  *slash_pos = '/';
+			  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+			}
+		      else
+			board_url[0] = '\0';
+		    }
+		}
+	    }
+#if ENABLE_REGEXP
+	  else if (tmp_pos[12] == '?')
+	    {
+	      /*
+	       * http://jbbs.shitaraba.com/computer/bbs/read.cgi?BBS=351&KEY=1040452877&START=109&END=109&NOFIRST=TRUE
+	       * URL
+	       */
+	      if (parse_cgi_query(tmp_pos + 13, &board_id, &thread_id,
+				  &from, &to))
+		{
+		  *tmp_pos = '\0';
+		  if (snprintf(board_url, PATH_MAX, "http://%s%s%s/",
+			       server, abs_path, board_id) >= PATH_MAX)
+		    board_url[0] = '\0';
+		}
+	    }
+#endif
+	}
+    }
+#if ENABLE_REGEXP
+  else if (strncmp(path_pos, "bbs/read.pl?", 12) == 0)
+    {
+      bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
+      if (parse_cgi_query(path_pos + 12, &board_id, &thread_id,
+			  &from, &to))
+	{
+	  if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server, board_id)
+	      >= PATH_MAX)
+	    board_url[0] = '\0';
+#if 0
+	  fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+	}
+    }
+  else if (strncmp(path_pos, "read.cgi?", 9) == 0)
+    {
+      bbs_type = OCHUSHA_BBS_TYPE_MITINOKU;
+      if (parse_cgi_query(path_pos + 9, &board_id, &thread_id,
+			  &from, &to))
+	{
+	  if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server, board_id)
+	      >= PATH_MAX)
+	    board_url[0] = '\0';
+#if 0
+	  fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+	}
+    }
+#endif
+  else if (strncmp(path_pos, "bbs/read.cgi", 12) == 0)
+    {
+      if (path_pos[12] == '/')
+	{
+	  char *slash_pos = strchr(path_pos + 13, '/');
+	  if (slash_pos != NULL)
+	    {
+	      int len = slash_pos - (path_pos + 12) -1;
+	      memcpy(tmp_buf, path_pos + 12, len);
+	      tmp_buf[len] = '\0';
+	      if (snprintf(board_url, PATH_MAX, "http://%s%s/", server,
+			   tmp_buf) < PATH_MAX)
+		{
+		  board_id = G_STRDUP(tmp_buf);
+		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+		  bbs_type = OCHUSHA_BBS_TYPE_JBBS;	/* ̤ƥ */
+		}
+	      else
+		board_url[0] = '\0';
+#if 0
+	      fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+	    }
+	}
+#if ENABLE_REGEXP
+      else if (path_pos[12] == '?')
+	{
+	  if (parse_cgi_query(path_pos + 13, &board_id, &thread_id,
+			      &from, &to))
+	    {
+	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
+			   board_id) < PATH_MAX)
+		bbs_type = OCHUSHA_BBS_TYPE_JBBS;
+	      else
+		board_url[0] = '\0';
+#if 0
+	      fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+	    }
+	}
+#endif
+    }
+  else
+    {
+      /* 2chäݤġġ*/
+      char *slash_pos = strchr(path_pos, '/');
+      if (slash_pos != NULL)
+	{
+	  slash_pos[0] = '\0';
+	  snprintf(board_url, PATH_MAX, "http://%s%s", server, abs_path);
+	  board_id = G_STRDUP(abs_path + 1);
+	  slash_pos[0] = '/';
+	  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+	  bbs_type = OCHUSHA_BBS_TYPE_2CH;
+	}
+#if 0
+      fprintf(stderr, "  board_url: %s\n", board_url);
+#endif
+    }
+
+ done:
+  if (server != NULL)
+    G_FREE(server);
+
+  if (abs_path != NULL)
+    G_FREE(abs_path);
+
+  if (board_url[0] == '\0')
+    {
+      /*  */
+      if (board_id != NULL)
+	{
+	  G_FREE(board_id);
+	  board_id = NULL;
+	}
+      if (thread_id != NULL)
+	{
+	  G_FREE(thread_id);
+	  thread_id = NULL;
+	}
+      bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+    }
+
+  if (board_url_p != NULL)
+    {
+      if (board_url[0] != '\0')
+	*board_url_p = G_STRDUP(board_url);
+      else
+	*board_url_p = NULL;
+    }
+
+  if (bbs_type_p != NULL)
+    *bbs_type_p = bbs_type;
+
+  if (board_id_p != NULL)
+    *board_id_p = board_id;
+  else if (board_id != NULL)
+    G_FREE(board_id);
+
+  if (thread_id_p != NULL)
+    *thread_id_p = thread_id;
+  else if (thread_id != NULL)
+    G_FREE(thread_id);
+
+  if (from_p != NULL)
+    *from_p = from;
+
+  if (to_p != NULL)
+    *to_p = to;
+
+  return board_url[0] != '\0';
+}
+
+
 OchushaAsyncBuffer *
 ochusha_utils_2ch_get_bbsmenu(OchushaNetworkBroker *broker)
 {
@@ -377,129 +963,6 @@ ochusha_utils_2ch_analyze_bbsmenu(OchushaBBSTable *table,
 }
 
 
-/*
- * url2chĤ⤷ϥؤURLȸʤƲϤ롣
- *
- * Ϸ̤Ȥơ
- * бĤɽOchushaBulletinBoardؤΥݥ󥿤*board_pˡ
- * ֹ*thread_number_p
- * ɽOchushaBBSThreadؤΥݥ󥿤*thread_pˡ
- * 쥹ֻ꤬硢ˡƬ*from_p*to_p˳Ǽ롣
- * ˲äơURLʸΥåɻʬʸΥԡؤΥݥ
- * *thread_part_p˳Ǽ롣
- *
- * caller*thread_partȤäfree뤳ȡ
- *
- * ǤˤĤƤ⡢Ϳ줿ݥ󥿤NULLξˤϷ̤ϳǼ
- * ʤ
- *
- * ϤˤTRUE֤URL2chĤ⤷ϥURL̵
- * ˤFALSE֤
- */
-gboolean
-ochusha_utils_2ch_parse_url(OchushaBBSTable *table, const char *url,
-			    OchushaBulletinBoard **board_p,
-			    unsigned int *thread_number_p,
-			    OchushaBBSThread **thread_p,
-			    unsigned int *from_p, unsigned int *to_p,
-			    char **thread_part_p)
-{
-  char tmp_buf[PATH_MAX];
-  char *server;
-  char *path;
-  char *base_path;
-  char *tmp_pos = NULL;
-  char tmp_char;
-  OchushaBulletinBoard *board = NULL;
-  OchushaBBSThread *thread = NULL;
-  int thread_number = 0;
-  int from = 0;
-  int to = 0;
-  int count;
-
-  gboolean result = FALSE;
-
-  g_return_val_if_fail(OCHUSHA_IS_BBS_TABLE(table) && url != NULL, FALSE);
-
-  server = ochusha_utils_url_extract_http_server(url);
-  path = ochusha_utils_url_extract_http_absolute_path(url);
-
-  if (path == NULL || strncmp(path, "/test/read.cgi/", 15) != 0)
-    goto done;
-
-  base_path = path + 15;
-  tmp_pos = strchr(base_path, '/');
-  if (tmp_pos == NULL)
-    goto done;
-
-  tmp_char = tmp_pos[1];
-  tmp_pos[1] = '\0';
-  if (snprintf(tmp_buf, PATH_MAX, "http://%s/%s", server, base_path)
-      >= PATH_MAX)
-    goto done;
-
-  board = ochusha_bbs_table_lookup_board_by_url(table, tmp_buf);
-  if (board != NULL)
-    result = TRUE;
-
-  tmp_pos[1] = tmp_char;
-
-  count = sscanf(tmp_pos + 1, "%u/%u-%u", &thread_number, &from, &to);
-
-  if (thread_number < 0)
-    {
-      /* ʬäȤ */
-      thread_number = 0;
-      from = 0;
-      to = 0;
-      result = FALSE;
-    }
-  else
-    {
-      if (board != NULL)
-	{
-	  snprintf(tmp_buf, PATH_MAX, "%u.dat", thread_number);
-	  thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
-								  tmp_buf);
-	}
-      if (from < 0)
-	{
-	  to = -from;
-	  from = 1;
-	}
-      else if (to < 0)
-	{
-	  to = -to;
-	}
-    }
-
- done:
-  if (server != NULL)
-    G_FREE(server);
-
-  if (path != NULL)
-    G_FREE(path);
-
-  if (result)
-    {
-      if (board_p != NULL)
-	*board_p = board;
-      if (thread_number_p != NULL)
-	*thread_number_p = thread_number;
-      if (thread_p != NULL)
-	*thread_p = thread;
-      if (from_p != NULL)
-	*from_p = from;
-      if (to_p != NULL)
-	*to_p = to;
-      if (thread_part_p != NULL)
-	*thread_part_p = G_STRDUP(tmp_pos + 1);
-    }
-
-  return result;
-}
-
-
 OchushaUtils2chPostResult
 ochusha_utils_2ch_try_post(OchushaBBSThread *thread, const char *message)
 {
