@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_jbbs.c,v 1.7 2003/11/20 03:14:37 fuyu Exp $
+ * $Id: ochusha_utils_jbbs.c,v 1.8 2003/11/21 21:30:03 fuyu Exp $
  */
 
 #include "config.h"
@@ -38,7 +38,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <iconv.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -48,14 +47,14 @@
 
 #if ENABLE_REGEXP
 # if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
+#  if HAVE_ONIG_ONIGURUMA_H
+#    include <onig/oniguruma.h>
 #    define ENABLE_HTML_TO_DAT_CONVERTER	1
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
+#  elif HAVE_ONIGURUMA_H
+#    include <oniguruma.h>
 #    define ENABLE_HTML_TO_DAT_CONVERTER	1
 #  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
+#    error "Use of oniguruma without oniguruma.h header isn't considered."
 #  endif
 # else
 #    define ENABLE_HTML_TO_DAT_CONVERTER	0
@@ -64,8 +63,9 @@
 
 
 #if ENABLE_HTML_TO_DAT_CONVERTER
-#define DEFAULT_LINE_BUFFER_SIZE	8192
-#define DEBUG_HTML_TO_DAT_CONVERTER	0
+#define DEFAULT_LINE_BUFFER_SIZE		8192
+#define DEBUG_HTML_TO_DAT_CONVERTER		1
+#define DEBUG_HTML_TO_DAT_CONVERTER_MOST	0
 
 
 typedef struct _HTML2DATConverterJobArgs
@@ -79,6 +79,59 @@ typedef struct _HTML2DATConverterJobArgs
 } HTML2DATConverterJobArgs;
 
 
+static char *title_pattern_string = NULL;
+static char *title_pattern_end = NULL;
+static char *jbbs_shitaraba_response_pattern_string = NULL;
+static char *jbbs_shitaraba_response_pattern_end = NULL;
+static char *machibbs_response_pattern_string = NULL;
+static char *machibbs_response_pattern_end = NULL;
+static char *mitinoku_response_pattern_string = NULL;
+static char *mitinoku_response_pattern_end = NULL;
+
+
+static void
+initialize_default_regexp_patterns(void)
+{
+  title_pattern_string = "<title>(.*)</title>";
+  title_pattern_end
+    = title_pattern_string + strlen(title_pattern_string);
+
+  /* "\314\276\301\260\241\247" == "̾" in EUC-JP */
+  /* "\305\352\271\306\306\374\241\247" == "" in EUC-JP */
+#if 0
+  jbbs_shitaraba_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) \314\276\301\260\241\247[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\305\352\271\306\306\374\241\247 ([^<]*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#else
+  /* ʤ"̾"ѥˤȥޥåʤġġ*/
+  jbbs_shitaraba_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) [^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\305\352\271\306\306\374\241\247 ([^<]*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#endif
+  jbbs_shitaraba_response_pattern_end
+    = (jbbs_shitaraba_response_pattern_string
+       + strlen(jbbs_shitaraba_response_pattern_string));
+
+  /* "\226\274\221\117\201\106" == "̾" in Shift_JIS */
+  /* "\223\212\215\145\223\372\201\106" == "" in Shift_JIS */
+#if 0
+  machibbs_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) \226\274\221\117\201\106[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\223\212\215\145\223\372\201\106 (.*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#else
+  machibbs_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) [^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\223\212\215\145\223\372\201\106 (.*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#endif
+  machibbs_response_pattern_end
+    = (machibbs_response_pattern_string
+       + strlen(machibbs_response_pattern_string));
+
+  /* "\314\276\301\260\241\247" == "̾" in EUC-JP */
+  /* "\305\352\271\306\306\374\241\247" == "" in EUC-JP */
+#if 0
+  mitinoku_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) \314\276\301\260\241\247[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\305\352\271\306\306\374\241\247(.*)<[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#else
+  mitinoku_response_pattern_string = "<[dD][tT]>([1-9][0-9]*) [^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\305\352\271\306\306\374\241\247(.*)<[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$";
+#endif
+  mitinoku_response_pattern_end
+    = (mitinoku_response_pattern_string
+       + strlen(mitinoku_response_pattern_string));
+}
+
+
 static void
 convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 {
@@ -91,82 +144,84 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   gboolean signal_result;
   unsigned int html_offset;
   unsigned int response_number;
-  regex_t title_pattern;
-  regex_t response_pattern;
-  iconv_helper *input_helper
-    = ochusha_bbs_thread_get_response_iconv_helper(thread);
-  iconv_helper *output_helper = NULL;	/* XXX: Τġ */
-  const char *html_encoding
-    = ochusha_bbs_thread_get_response_character_encoding(thread);
-  iconv_t input_converter = NULL;
-  iconv_t output_converter = NULL;
-  char output_encoding[1024];
   char *title = NULL;
   gboolean dump_html = FALSE;
+  regex_t *title_pattern = NULL;
+  regex_t *response_pattern = NULL;
+  RegRegion *match_region = NULL;
 
-  if (html_encoding == NULL)
-    goto terminated;
-
-  if (input_helper != NULL)
-    input_converter = iconv_open("UTF-8", html_encoding);
-  else
-    input_converter = iconv_open("UTF-8//IGNORE", html_encoding);
-
-  if (input_converter == NULL)
-    goto terminated;
-
-  if (snprintf(output_encoding, 1024, "%s//IGNORE", html_encoding) >= 1024)
-    goto terminated;
-
-  output_converter = iconv_open(output_encoding, "UTF-8");
-  if (output_converter == NULL)
-    goto terminated;
+  if (title_pattern_string == NULL)
+    initialize_default_regexp_patterns();
 
   switch (thread->board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      if (regcomp(&title_pattern, "<title>(.*)</title>", REG_EXTENDED))
+      if (regex_new(&title_pattern, title_pattern_string, title_pattern_end,
+		    REG_OPTION_EXTEND, REGCODE_EUCJP,
+		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
-      /* "\345\220\215\345\211\215\357\274\232" == "̾" */
-      /* "\346\212\225\347\250\277\346\227\245\357\274\232" == "" */
-      if (regcomp(&response_pattern, "<[dD][tT]>([1-9][0-9]*) \345\220\215\345\211\215\357\274\232[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\346\212\225\347\250\277\346\227\245\357\274\232 ([^<]*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$", REG_EXTENDED))
+      if (regex_new(&response_pattern, jbbs_shitaraba_response_pattern_string,
+		    jbbs_shitaraba_response_pattern_end,
+		    REG_OPTION_EXTEND, REGCODE_EUCJP,
+		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
+      fprintf(stderr, "title_pattern: \"%s\"\n", title_pattern_string);
+      fprintf(stderr, "response_pattern: \"%s\"\n",
+	      jbbs_shitaraba_response_pattern_string);
+#endif
       break;
 
     case OCHUSHA_BBS_TYPE_MACHIBBS:
-      if (regcomp(&title_pattern, "<title>(.*)</title>", REG_EXTENDED))
+      if (regex_new(&title_pattern, title_pattern_string, title_pattern_end,
+		    REG_OPTION_EXTEND, REGCODE_SJIS,
+		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
-      /* "\345\220\215\345\211\215\357\274\232" == "̾" */
-      /* "\346\212\225\347\250\277\346\227\245\357\274\232" == "" */
-      if (regcomp(&response_pattern, "<[dD][tT]>([1-9][0-9]*) \345\220\215\345\211\215\357\274\232[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\346\212\225\347\250\277\346\227\245\357\274\232 (.*)<[bB][rR]><[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$", REG_EXTENDED))
+        if (regex_new(&response_pattern, machibbs_response_pattern_string,
+		      machibbs_response_pattern_end,
+		      REG_OPTION_EXTEND, REGCODE_SJIS,
+		      REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
+      fprintf(stderr, "title_pattern: \"%s\"\n", title_pattern_string);
+      fprintf(stderr, "response_pattern: \"%s\"\n",
+	      machibbs_response_pattern_string);
+#endif
       break;
 
     case OCHUSHA_BBS_TYPE_MITINOKU:
-      if (regcomp(&title_pattern, "<title>(.*)</title>", REG_EXTENDED))
+      if (regex_new(&title_pattern, title_pattern_string, title_pattern_end,
+		    REG_OPTION_EXTEND, REGCODE_EUCJP,
+		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
-      /* "\345\220\215\345\211\215\357\274\232" == "̾" */
-      /* "\346\212\225\347\250\277\346\227\245\357\274\232" == "" */
-      if (regcomp(&response_pattern, "<[dD][tT]>([1-9][0-9]*) \345\220\215\345\211\215\357\274\232[^<]*(<[fF][oO][nN][tT][^>]*|<[aA][^>]*)>(.*)\346\212\225\347\250\277\346\227\245\357\274\232(.*)<[dD][dD]>(.*)<[bB][rR]><[bB][rR]>$", REG_EXTENDED))
+      if (regex_new(&response_pattern, mitinoku_response_pattern_string,
+		    mitinoku_response_pattern_end,
+		    REG_OPTION_EXTEND, REGCODE_EUCJP,
+		    REG_SYNTAX_POSIX_EXTENDED, NULL) != REG_NORMAL)
 	{
 	  fprintf(stderr, "invalid regular expression\n");
 	  goto terminated;
 	}
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
+      fprintf(stderr, "title_pattern: \"%s\"\n", title_pattern_string);
+      fprintf(stderr, "response_pattern: \"%s\"\n",
+	      mitinoku_response_pattern_string);
+#endif
       break;
 
     case OCHUSHA_BBS_TYPE_JBBS:
@@ -177,7 +232,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       goto terminated;
     }
 
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
   fprintf(stderr, "convert_html_to_dat called\n");
 #endif
 
@@ -202,6 +257,10 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   html_offset = 0;
   response_number = number_of_cached_responses + 1;
 
+  g_assert(title_pattern != NULL);
+  g_assert(response_pattern != NULL);
+  match_region = regex_region_new();
+
   ochusha_async_buffer_lock(html_buffer);
   while (TRUE)
     {
@@ -211,7 +270,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       char *eol_pos = NULL;
       char *previous_line = NULL;
 
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
       fprintf(stderr,
 	      "html_buffer_top=%p, html_buffer_pos=%p, rest_of_data=%d\n",
 	      html_buffer_top, html_buffer_pos, rest_of_data);
@@ -220,51 +279,61 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       while (rest_of_data > 0 && (eol_pos = memchr(html_buffer_pos, '\n',
 						   rest_of_data)) != NULL)
 	{
-	  char *html_line_buffer;
-	  int line_size = eol_pos - html_buffer_pos + 1;
-	  regmatch_t match[6];
-	  html_line_buffer = convert_string(input_converter, input_helper,
-					    html_buffer_pos, line_size);
+	  int line_size = eol_pos - html_buffer_pos;
+	  char *html_line_buffer = G_STRNDUP(html_buffer_pos, line_size);
+	  char *html_line_buffer_tail = html_line_buffer + line_size;
+	  line_size++;
+
 	  if (previous_line != NULL)
 	    {
 	      char *tmp_line = g_strconcat(previous_line, html_line_buffer,
 					   NULL);
 	      g_free(previous_line);
 	      previous_line = html_line_buffer;
+	      html_line_buffer_tail = tmp_line + strlen(tmp_line);
 	      html_line_buffer = tmp_line;
 	    }
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "html_buffer_pos=%p, eol_pos=%p, line_size=%d\n",
 		  html_buffer_pos, eol_pos, line_size);
 	  fprintf(stderr, "html_line: \"%s\"\n", html_line_buffer);
 #endif
 
+	  regex_region_clear(match_region);
 	  if (response_number == 1 && title == NULL
-	      && regexec(&title_pattern, html_line_buffer, 1, match, 0) == 0)
+	      && regex_search(title_pattern,
+			      html_line_buffer, html_line_buffer_tail,
+			      html_line_buffer, html_line_buffer_tail,
+			      match_region, 0) >= 0)
 	    {
-	      char *title_head = html_line_buffer + match[1].rm_so;
-	      int title_len = match[1].rm_eo - match[1].rm_so;
+	      char *title_head = html_line_buffer + match_region->beg[1];
+	      int title_len = match_region->end[1] - match_region->beg[1];
 	      title = G_STRNDUP(title_head, title_len);
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
+	      fprintf(stderr, "title: %s\n", title);
+#endif
 	    }
 	  else if (dump_html)
 	    {
 	      fprintf(stderr, "html_line: \"%s\"\n", html_line_buffer);
 	    }
-	  else if (regexec(&response_pattern,
-			   html_line_buffer, 5, match, 0) == 0)
+	  else if (regex_search(response_pattern,
+				html_line_buffer, html_line_buffer_tail,
+				html_line_buffer, html_line_buffer_tail,
+				match_region, 0) >= 0)
 	    {
 	      int res_num;
-	      char *tag_head = html_line_buffer + match[2].rm_so;
-	      int tag_len = match[2].rm_eo - match[2].rm_so;
+	      char *tag_head = html_line_buffer + match_region->beg[2];
+	      int tag_len = match_region->end[2] - match_region->beg[2];
 
-	      char *name_head = html_line_buffer + match[3].rm_so;
-	      int name_len = match[3].rm_eo - match[3].rm_so;
+	      char *name_head = html_line_buffer + match_region->beg[3];
+	      int name_len = match_region->end[3] - match_region->beg[3];
 
-	      char *date_head = html_line_buffer + match[4].rm_so;
-	      int date_len = match[4].rm_eo - match[4].rm_so;
+	      char *date_head = html_line_buffer + match_region->beg[4];
+	      int date_len = match_region->end[4] - match_region->beg[4];
 
-	      char *body_head = html_line_buffer + match[5].rm_so;
-	      int body_len = match[5].rm_eo - match[5].rm_so;
+	      char *body_head = html_line_buffer + match_region->beg[5];
+	      int body_len = match_region->end[5] - match_region->beg[5];
 
 	      char *mailto_head = NULL;
 	      int mailto_len = 0;
@@ -288,9 +357,9 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 		    }
 		}
 
-	      sscanf(html_line_buffer + match[1].rm_so, "%d", &res_num);
+	      sscanf(html_line_buffer + match_region->beg[1], "%d", &res_num);
 
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	      fprintf(stderr, "res_num=%d : response_number=%d\n",
 		      res_num, response_number);
 #endif
@@ -298,7 +367,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	      while (res_num > response_number)
 		{
 		  /* ŪDATƤ */
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 		  fprintf(stderr, "breaking DAT for: \"%s\"\n",
 			  html_line_buffer);
 #endif
@@ -420,32 +489,16 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #undef APPEND_TEXT_TO_DAT_LINE_BUFFER
 		  if (dat_line_buffer_rest > 0)
 		    {
-		      char *dat_line
-			= convert_string(output_converter, output_helper,
-					 dat_line_buffer,
-					 dat_line_buffer_pos - dat_line_buffer);
-		      if (dat_line != NULL)
-			{
-			  ochusha_async_buffer_append_data(dat_buffer,
-							   dat_line,
-							   strlen(dat_line),
+		      ochusha_async_buffer_append_data(dat_buffer,
+				dat_line_buffer,
+				dat_line_buffer_pos - dat_line_buffer,
 				"ochusha_utils_jbbs.c: convert_html_to_dat()");
-#if DEBUG_HTML_TO_DAT_CONVERTER
-			  fprintf(stderr, "DAT: %s", dat_line);
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
+		      {
+			dat_line_buffer_pos[0] = '\0';
+			fprintf(stderr, "DAT: %s\n", dat_line_buffer);
+		      }
 #endif
-			  G_FREE(dat_line);
-			}
-		      else
-			{
-			  /* ŪDATƤ */
-#if DEBUG_HTML_TO_DAT_CONVERTER
-			  fprintf(stderr,
-				  "convert_string returns NULL for \"%s\"\n",
-				  dat_line_buffer);
-#endif
-			  ochusha_async_buffer_append_data(dat_buffer, "\n", 1,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
-			}
 		    }
 		  else
 		    {
@@ -503,7 +556,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
       if (html_buffer->fixed)
 	{
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "conversion done.\n");
 #endif
 	  g_signal_emit_by_name(G_OBJECT(args->broker), "access_completed",
@@ -520,7 +573,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       if (!ochusha_async_buffer_wait(html_buffer,
 				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
 	{
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
 #endif
 	  g_signal_emit_by_name(G_OBJECT(args->broker), "access_terminated",
@@ -532,7 +585,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	    }
 	  break;
 	}
-#if DEBUG_HTML_TO_DAT_CONVERTER
+#if DEBUG_HTML_TO_DAT_CONVERTER_MOST
       else
 	fprintf(stderr, "wait signaled.\n");
 #endif
@@ -574,10 +627,12 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 			   "ochusha_utils_jbbs.c: convert_html_to_dat()");
 
  terminated:
-  if (input_converter != NULL)
-    iconv_close(input_converter);
-  if (output_converter != NULL)
-    iconv_close(output_converter);
+  if (title_pattern != NULL)
+    regex_free(title_pattern);
+  if (response_pattern != NULL)
+    regex_free(response_pattern);
+  if (match_region != NULL)
+    regex_region_free(match_region, 1);
   g_object_unref(G_OBJECT(dat_buffer));
   g_object_unref(G_OBJECT(args->broker));
 }
@@ -674,6 +729,7 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 	  close(fd);
 	}
     }
+
   /* λǼεDATե뤬ѲǽʾΤdat_bufferNULL */
 
   switch (thread->board->bbs_type)
@@ -685,33 +741,24 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 		   ochusha_board_2ch_get_read_cgi_url(board_2ch),
 		   thread->board->id, thread->id,
 		   number_of_cached_responses + 1,
-		   number_of_cached_responses == 0 ? "" : "NOFIRST=TRUE")
+		   number_of_cached_responses == 0 ? "" : "&NOFIRST=TRUE")
 	  >= PATH_MAX)
 	return dat_buffer;
       break;
 
-#if 1
-      /*
-       * number_of_cached_responsesĤȤɤ򤷤
-       * ΤstꤹȤ50쥹ʬ֤äƤʤ̣
-       */
     case OCHUSHA_BBS_TYPE_MITINOKU:
-      if (snprintf(cgi_url, PATH_MAX, "%s?bbs=%s&key=%s",
-		   ochusha_board_2ch_get_read_cgi_url(board_2ch),
-		   thread->board->id, thread->id) >= PATH_MAX)
+      if (thread->number_of_responses_on_server <= number_of_cached_responses)
 	return dat_buffer;
-      break;
-#else
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      if (snprintf(cgi_url, PATH_MAX, "%s?bbs=%s&key=%s&st=%d%s",
+
+      if (snprintf(cgi_url, PATH_MAX, "%s?bbs=%s&key=%s&st=%d&to=%d%s",
 		   ochusha_board_2ch_get_read_cgi_url(board_2ch),
 		   thread->board->id, thread->id,
 		   number_of_cached_responses + 1,
-		   number_of_cached_responses == 0 ? "" : "nofirst=true")
+		   thread->number_of_responses_on_server,
+		   number_of_cached_responses == 0 ? "" : "&nofirst=true")
 	  >= PATH_MAX)
 	return dat_buffer;
       break;
-#endif
 
     default:	/* ̤ݡ */
       return dat_buffer;
